"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const htmlparser2_1 = require("htmlparser2");
const js_beautify_1 = require("js-beautify");
const path_1 = require("path");
const util_1 = require("util");
const component_parser_1 = require("./component-parser");
function loader(content, map) {
    const callback = this.async();
    parseXMLTree.bind(this)(content).then((output) => {
        callback(null, output, map);
    }).catch((err) => {
        callback(err);
    });
}
exports.default = loader;
async function parseXMLTree(content) {
    const { appPath, platform } = this.getOptions();
    const moduleRelativePath = (0, path_1.relative)(appPath, this.resourcePath);
    const resolveAsync = (0, util_1.promisify)(this.resolve);
    const componentParser = new component_parser_1.ComponentParser(moduleRelativePath, platform);
    let needsCompilation = true;
    const xmlParser = new htmlparser2_1.Parser({
        onopentag(tagName, attributes) {
            componentParser.handleOpenTag(tagName, attributes);
        },
        onprocessinginstruction(name) {
            if (name == '?xml') {
                needsCompilation = false;
                xmlParser.reset();
            }
        },
        onclosetag(tagName) {
            componentParser.handleCloseTag(tagName);
        },
        onerror(err) {
            throw err;
        }
    }, {
        xmlMode: true
    });
    xmlParser.write(content);
    xmlParser.end();
    if (!needsCompilation) {
        // escape special whitespace characters
        // see: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/JSON/stringify#Issue_with_plain_JSON.stringify_for_use_as_JavaScript
        const xml = JSON.stringify(content).replace(/\u2028/g, '\\u2028').replace(/\u2029/g, '\\u2029');
        return `const RAW_XML_CONTENT = ${xml};
    export default RAW_XML_CONTENT;`;
    }
    // XML parser does not work asynchronously, so we resolve requested modules once everything is done
    await Promise.all(componentParser.getResolvedRequests().map(request => resolveAsync(this.context, request)));
    return (0, js_beautify_1.js)(componentParser.getResult(), {
        indent_size: 2
    });
}
//# sourceMappingURL=index.js.map